﻿//-----------------------------------------------------------------------
// <copyright file="Engine.cs" company="Mad Wonder LLC">
//     Copyright (c) Mad Wonder LLC. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace MadWonder.Tutorial02 {
    public class Engine : MonoBehaviour
    {
        public Material targetMaterial;

        [ColorUsage(false)]
        public Color32 backgroundColor = new Color32(184, 184, 184, 255);

        [HideInInspector]
        public Texture2D canvasTexture;

        [Range(1.0f, 4096.0f)]
        public int pixelWidth = 320;

        [Range(1.0f, 2160.0f)]
        public int pixelHeight = 180;

        [Range(1.0f, 1024.0f)]
        public int numberOfLayers = 8;

        [Range(1.0f, 1024.0f)]
        public int numberOfSprites = 32;

        [Range(1.0f, 1024.0f)]
        public int numberOfAnimations = 64;

        public bool displayQuad;

        private EngineManager engineRef;
        private byte[] renderPixels;

        public void Awake()
        {
            this.engineRef = new EngineManager(this.pixelWidth, this.pixelHeight, this.numberOfLayers, this.numberOfSprites, this.numberOfAnimations);
            this.canvasTexture = new Texture2D(this.pixelWidth, this.pixelHeight, TextureFormat.BGRA32, false);

            if (this.displayQuad) {
                this.SetupDisplayQuad();
            }

            if (this.targetMaterial != null) {
                this.targetMaterial.mainTexture = this.canvasTexture;
            }

            int byteCount = this.pixelWidth * this.pixelHeight * 4;
            this.renderPixels = new byte[byteCount];

            this.engineRef.SetRenderTarget(this.renderPixels);
            this.engineRef.SetBackgroundColor(this.backgroundColor);
            this.engineRef.UpdateFrame();

            this.canvasTexture.LoadRawTextureData(this.renderPixels);
            this.canvasTexture.Apply();
        }

        void Start() { }

        void Update()
        {
            this.engineRef.UpdateFrame();
            this.canvasTexture.LoadRawTextureData(this.renderPixels);
            this.canvasTexture.Apply();
        }

        private void SetupDisplayQuad() {
            GameObject freshQuad = GameObject.CreatePrimitive(PrimitiveType.Quad);
            freshQuad.name = "RenderQuad";
            freshQuad.transform.parent = this.transform;
            freshQuad.transform.localPosition = new Vector3(0.0f, 0.0f, 0.0f);

            Renderer quadAlterRender = freshQuad.GetComponent<Renderer>();
            Shader freshUnlit = Shader.Find("Unlit/Texture");
            Material alterMat = new Material(freshUnlit) { mainTexture = this.canvasTexture };
            quadAlterRender.material = alterMat;

            float targetRatio = (float)this.pixelWidth / (float)this.pixelHeight;
            float verticalScale = 1.0f;
            Camera snatchCam = GameObject.FindWithTag("MainCamera").GetComponent<Camera>();
            if (snatchCam != null && snatchCam.orthographic)
            {
                verticalScale = snatchCam.orthographicSize * 2.0f;
            }
            freshQuad.transform.localScale = new Vector3(verticalScale * targetRatio, verticalScale, 1.0f);
        }
    }
}
