﻿//-----------------------------------------------------------------------
// <copyright file="EngineManager.cs" company="Mad Wonder LLC">
//     Copyright (c) Mad Wonder LLC. All rights reserved.
// </copyright>
//-----------------------------------------------------------------------

using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace MadWonder.Tutorial02 {
    public class EngineManager : IDisposable
    {
        private int width;
        private int height;
        private int layers;
        private int sprites;
        private int animations;

        private IntPtr engineHandle;

        private bool disposed;
        private int frameCounter;

        public EngineManager(int inputWidth, int inputHeight, int inputLayers, int inputSprites, int inputAnimations) {
            this.width = inputWidth;
            this.height = inputHeight;
            this.layers = inputLayers;
            this.sprites = inputSprites;
            this.animations = inputAnimations;

            this.frameCounter = 0;
            this.disposed = false;

            this.engineHandle = TilengineMapping.TLN_Init(this.width, this.height, this.layers, this.sprites, this.animations);
        }

        public virtual void UpdateFrame() {
            bool contextSet = TilengineMapping.TLN_SetContext(this.engineHandle);
            if (contextSet) {
                TilengineMapping.TLN_UpdateFrame(this.frameCounter);
                this.frameCounter++;
            }
        }

        public virtual void SetRenderTarget(byte[] pixelArray) {
            bool contextSet = TilengineMapping.TLN_SetContext(this.engineHandle);
            if (contextSet) {
                TilengineMapping.TLN_SetRenderTarget(pixelArray, this.width * 4);
            }
        }

        public virtual void SetBackgroundColor(Color32 colorToSet) {
            bool contextSet = TilengineMapping.TLN_SetContext(this.engineHandle);
            if (contextSet) {
                TilengineMapping.TLN_SetBGColor(colorToSet.r, colorToSet.g, colorToSet.b);
            }
        }

        public void Dispose() {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        protected virtual void Dispose(bool manualDispose) {
            if (!this.disposed) {
                if (manualDispose) {
                    // Here's where you put any code for clearing out regular managed resources.
                }
                bool contextSet = TilengineMapping.TLN_SetContext(this.engineHandle);
                if (contextSet) {
                    bool engineCleared = TilengineMapping.TLN_DeleteContext(this.engineHandle);
                    if (engineCleared) {
                        this.engineHandle = IntPtr.Zero;
                    }
                }
                this.disposed = true;
            }
        }

        ~EngineManager() {
            this.Dispose(false);
        }
    }
}
